function [loss] = obj_CUR(X)  

  [p,n] = size(X);
  loss.obj = @obj_only;
  loss.obj_cone_2d = @obj_cone_2d;
  loss.update_ctx_2d_pre = @update_ctx_2d_pre;
  loss.update_ctx_2d_post = @update_ctx_2d_post;
  loss.obj_cone_total = @obj_cone_total;
  loss.update_ctx_total = @update_ctx_total;
  loss.Xnorm = norm(X,'fro');
  loss.ctx = struct('c', [], ...
                    'Q', [], ...
                    'R', [], ...
                    'd', 0, ...
                    'PRprod', 0, ...
                    'Rnorm', 0, ...
                    'Qnorm', 0);

  function [f, G] = obj_only(W)
    
    if isvec(W)
      W_is_vec = true;
      W = reshape(W, n, p);
    else
      W_is_vec = false;
    end

    G = X*W*X - X;
    f = 0.5*norm(G,'fro')^2;
    G = X'*G*X';

    %   denom = max(size(X));
    %   fprintf('In obj_CUR: f = %g, f/denom = %g\n', f, f/denom);
    %   f = f/denom;    G = G/denom;

    if W_is_vec,   G = G(:);    end
  end

  function [f, g] = obj_cone_total(weights, loss)
    g = loss.ctx.R*weights;
    f = loss.Xnorm^2/2 - weights'*loss.ctx.c + weights'*g/2;
    g = g - loss.ctx.c;
  end

  function res = update_ctx_total(ctx, basis, supp)
    
    w = zeros(p*n, 1);
    w(supp) = basis;
    Q = X*reshape(w, n, p)*X;
    res.Q = [ctx.Q; Q];
    res.c = [ctx.c; sum(sum(Q .* X))];
    num_basis = length(ctx.c);
    v = zeros(num_basis, 1);    
    for i = 1 : num_basis
      v(i) = sum(sum(Q .* ctx.Q((i-1)*p+1:i*p,:)));
    end
    res.R = [ctx.R v;
             v', sum(sum(Q.*Q))];    
  end

  function [f, g] = obj_cone_2d(ab, loss)
    a = ab(1);    b = ab(2);
    c = loss.ctx.c;   d = loss.ctx.d;
    f = loss.Xnorm^2/2 - a*c - b*d ...
          + 0.5*((a*loss.ctx.Qnorm)^2 + (b*loss.ctx.Rnorm)^2 ...
                  + 2*a*b*loss.ctx.QRprod);
    g = [a*loss.ctx.Qnorm^2 + b*loss.ctx.QRprod - c;
         b*loss.ctx.Rnorm^2 + a*loss.ctx.QRprod - d];
       
    if f < 0
      disp('a');
    end
  end

  function res = update_ctx_2d_post(ctx, ab)
    a = ab(1);    b = ab(2);
    res.Q = a*ctx.Q + b*ctx.R;
    res.c = a*ctx.c + b*ctx.d;
    res.Qnorm = sqrt(a^2*ctx.Qnorm^2 + b^2*ctx.Rnorm^2 + 2*a*b*ctx.QRprod);
  end

  function res = update_ctx_2d_pre(ctx, basis, supp)
    w = zeros(p*n, 1);
    w(supp) = basis;
    res.R = X*reshape(w, n, p)*X;
    res.d = sum(sum(X .* res.R));
    res.Rnorm = norm(res.R, 'fro');    
    if isempty(ctx.Q) % first iteration
      res.QRprod = 0;
      res.Qnorm = 0;
      res.Q = zeros(p,n);
      res.c = 0;
    else
      res.QRprod = sum(sum(res.R .* ctx.Q));
      res.Q = ctx.Q;
      res.Qnorm = ctx.Qnorm;
      res.c = ctx.c;
    end
  end

end
