% Compute the logistic loss for multi-class classification, and its gradient 

function [f, G, S] = logistic_multitask(dataset, W, flag)
            
  t=cputime;
  if nargin < 3, flag = [];   end
  
  if strcmp(flag, 'fdual')  % dual objective evaluated at X        
    error('fdual not implemented yet for logistic_multitask');    
%     f = 0;
%     return;
  end
  
  if strcmp(flag, 'dirHes')
    f = []; G = [];    
    model = W{1};
    searchDir = W{2};   % = U B V'
    searchDir.U = searchDir.U / dataset.num_ex;
    searchDir.V = searchDir.V / dataset.num_ex;
    searchDir.B = searchDir.B / dataset.num_ex;
    S = zeros(dataset.num_fea, dataset.num_task);
    for i = 1 : dataset.num_task
      X = dataset.x{i};
      Xtrans = X';
      S(:,i) = X*(Xtrans*(model.U*(model.B*(searchDir.V(i,:)')))) ...
                + X*(Xtrans*(searchDir.U*(model.B*(model.V(i,:)')))) ...
                + X*(Xtrans*(model.U*(searchDir.B*(model.V(i,:)'))));
    end
    return;
  end

  is_eval = strcmp(flag, 'eval');
  
  f = 0;
  if ~is_eval
    G = zeros(dataset.num_fea, dataset.num_task);
  end
  if isa(W, 'cell')
    Utrans = W{1}';
  end
  for i = 1 : dataset.num_task
    if isa(W, 'cell')
      output = W{2}(i,:) * (Utrans * dataset.x{i});
    else
      output = W(:, i)' * dataset.x{i};
    end
    diff = output' - dataset.y{i};   % #ex dim column vector
    f = f + norm(diff)^2;
    if ~is_eval
      G(:, i) = dataset.x{i} * (diff / dataset.num_ex);
    end
  end
  
  f = f/(2*dataset.num_ex);  
  
  global feval_logistic time_logistic
  time_logistic = time_logistic + (cputime - t);
  feval_logistic = feval_logistic + 1;

end
