% Compute the robust loss for multiview learning

function res = robust_multiview(data, par)

  if nargin < 3, flag = [];   end
   
  sigma = par.prob_par.sigma;
  w1 = par.prob_par.w1;
  w2 = par.prob_par.w2;
  m1 = data.m1;

  res.obj_x = @obj_x;
  res.obj_y = @obj_y;
  res.obj = @obj;
  res.eval = @eval;  
  res.eval_Y = @eval_Y;  

  function [f, G] = obj_x(W)
    [f, G] = smooth_L11(W, data.X(1:m1,:), sigma);
  end
    
  function [f, G] = obj_y(W)
    [f, G] = smooth_L11(W, data.X(1+m1:end,:), sigma);
  end

  function [f, G] = obj(W)
    [f1, G1] = smooth_L11(W(1:m1,:), data.X(1:m1,:), sigma);
    [f2, G2] = smooth_L11(W(1+m1:end,:), data.X(1+m1:end,:), sigma);

    f = w1*f1 + w2*f2;
    G = [w1*G1; w2*G2];
  end

  function res = eval(W)
    res.l1 = sum(sum(abs(W - [data.X1; data.Y1])));
    res.l2 = sum(sum((W - [data.X1; data.Y1]).^2));
  end

  function res = eval_Y(W)
%     res.l1 = sum(sum(abs(W(1+m1:end,:) - data.Y1)));
    res.l1 = norm(data.Y1, 'fro')^2 / norm(W(1+m1:end,:)-data.Y1, 'fro')^2;
    res.l2 = sum(sum((W(1+m1:end,:) - data.Y1).^2));
  end

end
