function [sol, obj, iter, msg, outputs] = ...
          multiview_pboost(lambda, X0, evaluator, opts)

% Solve  Min_X  func(X) + lambda * ||X||_*

m = size(X0, 1);
n = size(X0, 2);
k = opts.init_rank;  % Initialize U and V with rank k

U = randn(m, k);
V = randn(k, n);

make_output = (nargout == 5);

num_f1 = opts.num_f1;
num_f2 = opts.num_f2;
l = num_f1 + num_f2;
num_class = opts.num_class;
w1 = opts.weight_X1;
w2 = opts.weight_X2;
w3 = opts.weight_Y;
w = [w1, w2, w3];
ub1 = opts.ub(1);
ub2 = opts.ub(2);
assert(m == num_f1 + num_f2 + num_class);

if make_output
  outputs = [];
  outputs.obj_list = zeros(opts.max_iter, 1);
  outputs.time_list = zeros(opts.max_iter, 1);
  outputs.loss_list = zeros(opts.max_iter, 1);
  outputs.perf_list = zeros(opts.max_iter, 1);  
end

total_time = 0;     % Total time consumed by the algorithm
local_search_time = 0;  % Time spend on local search

global feval_logistic
global time_logistic

feval_logistic = 0;
time_logistic = 0;

perf = 0;
for i = 1 : opts.max_iter
      
  t1 = cputime;
  k = size(U, 2);
    
  if opts.use_local     
    t2 = cputime;
    nel = k*(m+n);
    ub = Inf(nel, 1);
    
    % local search    
    [UV, obj, iter, feval, msg] = lbfgsb([U(:); V(:)], -ub, ub, @obj_UV, ...
                                              [], [], opts.lbfgsb_in);
    U = reshape(UV(1:k*m), [m, k]);
    V = reshape(UV(1+k*m:end), [k, n]);
    t2 = cputime - t2;
    local_search_time = local_search_time + t2;
%     fprintf('\n');
  end
    
  norm_U = max([sqrt(sum(U(1:num_f1,:).^2))/ub1
                sqrt(sum(U(1+num_f1:l,:).^2))/ub2
                abs(U(l+1:end,:))])';
  norm_V = sqrt(sum(V.*V, 2));
  norm_UV = sqrt(norm_U .* norm_V);
  idx = norm_UV > 1e-6;
  nidx = sum(idx);
  if nidx < length(idx)
    U = U(:, idx);    V = V(idx, :);
    norm_U = norm_U(idx);    norm_V = norm_V(idx);    norm_UV = norm_UV(idx);
  end
  
  if opts.use_local
    loss = obj - 0.5*lambda*(norm_U'*norm_U + norm_V'*norm_V);
  end
  
  real_obj = loss + lambda * sum(norm_UV.^2);
  if opts.use_local || i > 1    
    fprintf('iter=%d, loss = %g, obj=%g, feval=%d, teval=%g, k=%d, time=%g, ls_time=%g, msg=%s\n', ...
          i, loss, real_obj, feval_logistic, time_logistic, k, total_time, local_search_time, msg2str(msg, 'lbfgsb'));
  end
  
  if nidx > 0
    sk = 0.5*(norm_U'*norm_U + norm_V'*norm_V); Xk = U*V;    
  else
    sk = 0; Xk = zeros(m, n);
  end

  if i > 1 && abs(pre_obj-obj) / min(abs([pre_obj, obj])) < opts.rtol
    msg = 'Stop with small relative change';    
%     disp(msg);
    break;
  else
    pre_obj = obj;
  end

  [u, v] = get_basis(Xk, opts);  
  
  norm_u = max([sqrt(sum(u(1:num_f1).^2))/ub1
                sqrt(sum(u(1+num_f1:l).^2))/ub2
                abs(u(l+1:end))])';
  
  % line search
  [weights, obj, iter, feval, msg] = lbfgsb([1; 0], [0; 0], [inf; inf], @obj_ls, ...
                                            [], [], opts.lbfgsb_in);
  fprintf('obj = %g, nu = %g, nv = %g, msg = %s\n', obj, norm_u, norm(v), msg2str(msg, 'lbfgsb'));

  loss = obj - lambda*(sk*weights(1) + weights(2));
  weights = sqrt(weights);
  if nidx > 0
      U = [U .* repmat(weights(1)*(norm_UV./norm_U)', [m, 1]), weights(2) * u];
      V = [V .* repmat(weights(1)*norm_UV./norm_V, [1, n]); weights(2) * v']; 
  else
      U = weights(2) * u;
      V = weights(2) * v'; 
  end
  
  t1 = cputime - t1;  total_time = total_time + t1;
  
  if ~isempty(evaluator)
    perf = evaluator(U);
  end
  
  if make_output
    outputs.obj_list(i) = obj;
    outputs.time_list(i) = total_time;
    outputs.loss_list(i) = loss;
    outputs.perf_list(i) = perf;
  end
end

if make_output
  outputs.obj_list = outputs.obj_list(1:i-1);
  outputs.time_list = outputs.time_list(1:i-1);
  outputs.loss_list = outputs.loss_list(1:i-1);
  outputs.perf_list = outputs.perf_list(1:i-1);
end

sol = U*V;
iter = i;
if i == opts.max_iter
  msg = 'Stop with max iteration';
end
                                          
  function [f, g] = obj_UV(X)
    r = length(X) / (m+n);
    UU = reshape(X(1:r*m), [m, r]);
    VV = reshape(X(1+r*m:end), [r, n]);
    [G1, G2, grads, f] = comp_grad_mv(UU*VV, opts);
    
    n1 = sum(UU(1:num_f1,:).^2);
    n2 = sum(UU(num_f1+1:l,:).^2);
    nsq_list = UU(l+1:end,:).^2;
    
    f = w*f + 0.5*lambda*(norm(VV,'fro')^2+sum(max([n1/ub1^2; n2/ub2^2; nsq_list])));
    
%     fprintf('f = %g\n', f);
    
    G = [G1*w1; G2*w2; grads*w3];
    range1 = 1:num_f1;
    range2 = num_f1+1:l;
    
    G_U = zeros(m, r); 
    cnt = zeros(3, 1);
    for j = 1 : r
      [nsq_max, nsq_idx] = max(nsq_list(:,j));
      if n1(j) >= max(n2(j), nsq_max)
        G_U(range1, j) = (lambda/ub1^2)*UU(range1, j);
        cnt(1) = cnt(1) + 1;
      elseif n2(j) >= max(n1(j), nsq_max)
        G_U(range2, j) = (lambda/ub2^2)*UU(range2, j);
        cnt(2) = cnt(2) + 1;
      else
        range3 = l+nsq_idx(1);
        G_U(range3, j) = lambda*UU(l+nsq_idx(1), j);
        cnt(3) = cnt(3) + 1;
      end
    end
%     fprintf('(%d, %d, %d)', cnt(1), cnt(2), cnt(3));
    g = [vec(G*VV'+G_U); vec(UU'*G+lambda*VV)];
  end
  
  function [f, g] = obj_ls(x)
    Y = x(1)*Xk + (x(2)*u)*v';
    [G1, G2, grads, f] = comp_grad_mv(Y, opts);
    f = w*f + lambda*(sk*x(1) + x(2));
    G = [G1*w1; G2*w2; grads*w3];
    g = [sum(sum(G.*Xk))+lambda*sk; u'*G*v+lambda];    
  end

end
