function [sol, obj, res] = ...
          solve_r1d(objf, lambda, X0, evalf, ctx_total, opts)

  % Solve  Min_X  objf(X) + lambda * ||X||_*
% ctx_total = [];

  [m, n] = size(X0);
%   k = opts.init_rank;  % Initialize by U, V with k dimensions
  k = 0;

  U = randn(m, k);  
  V = randn(k, n);
  theta = ones(k, 1);
  
  if k > 0
    U = U ./ repmat(sqrt(sum(U.^2)), m, 1);
    V = V ./ repmat(sqrt(sum(V.^2, 2)), 1, n);
  end

  delta = opts.tol / opts.L;
  perf = 0;
  time_list = [];
  time_polar = [];
  msg = 'Running';
  break_flag = 0;
  par = opts.ori_par;
  
  res = struct('obj_list', [], ...
               'loss_list', [], ...
               'perf_list', [] ...
               );

  global feval_logistic time_logistic

  feval_logistic = 0;
  time_logistic = 0;
    
  for i = 1 : opts.max_iter

    t1 = cputime;

    if size(U, 2) > 0
      UU = U .* repmat(theta', [m, 1]);
      [~, G] = objf([{UU}, {V'}]);
    else
      [~, G] = objf([{zeros(m, 1)}, {zeros(n,1)}]); 
    end

    tt = cputime;
    [u, s, v] = lansvd(@(X)cell_times_mat(G,X), @(X)cell_trans_times_mat(G,X), m, n, 1);
    v = -v;
    time_polar = [time_polar; cputime - tt];
    
    g = lambda - s;

%     if g <= - opts.tol / 2
%       U = [U, u];    V = [V; v'];
%       theta = [theta; delta];
%       msg = 'keep on';
%     else
%       % Compute gradient wrt theta_t
%       if size(U, 2) > 0
%         H = cell_times_mat(G, V');
%         grad_theta = sum(U .* H);
%         if max(abs(grad_theta + lambda)) < opts.tol
%           break;
%         end
%       end

      if ~isempty(ctx_total)
        ctx_total.ctx = ctx_total.update_context(ctx_total.ctx, U, V', u, v);
      end
      
      U = [U, u];     V = [V; v'];
      k = size(U, 2);
      [theta, obj, iter, feval, msg] = lbfgsb([theta; 1], zeros(k, 1), Inf(k, 1), ...
                                        @obj_theta, [], [], opts.lbfgsb_in);
      idx = theta > -1e-5;
      U = U(:, idx);  V = V(idx, :);    theta = theta(idx);
%     end

    time_list = [time_list; cputime - t1];

    k = size(U, 2);
    UU = U .* repmat(theta', [m, 1]);
    cur_loss = objf([{UU}, {V'}]);
    cur_obj = cur_loss + lambda * sum(theta);
    res.loss_list = [res.loss_list; cur_loss];    
    res.obj_list = [res.obj_list; cur_obj];
    
    if ~isempty(evalf)
      perf = evalf([{UU}, {V'}]);
      res.perf_list = [res.perf_list; perf];
    end

    if i == opts.max_iter,  msg = 'Stop with max iteration';    end
        
    fprintf('iter=%d, loss = %g, obj=%g, perf=(%g, %g), time=%g, feval=%d, teval=%g, k=%d, tpolar=%g, msg=%s\n', ...
            i, cur_loss, cur_obj, perf.l1, perf.l2, sum(time_list), ...
            feval_logistic, time_logistic, k, ...
            sum(time_polar), ...
            msg2str(msg, 'lbfgsb'));    
    
    if i > 1 && abs(pre_obj - cur_obj) / min(abs([pre_obj, cur_obj])) < opts.rtol
      msg = 'Stop with small relative change';    
      break_flag = 1;
    elseif isfield(opts, 'max_time') && sum(time_list) > opts.max_time
      msg = 'Stop by max_time';
      break_flag = 1;
    else
      pre_obj = cur_obj;
    end
    
    res.time_list = cumsum(time_list);
    res.time_polar = cumsum(time_polar);    
    res.msg = msg;
    res.iter = i;
    
    if isfield(opts, 'res_fname')
      save(opts.res_fname, 'res', 'par');
    end
    
    if break_flag == 1, break;    end
  end

  sol = (U.*repmat(theta', [m,1]))*V;
  obj = res.obj_list(end);
  

    function [f, g] = obj_theta(x)
      if isempty(ctx_total)
        UU = U .* repmat(x', [m, 1]);
        [f, G] = objf([{UU}, {V'}]);

        f = f + lambda * sum(x);
        H = cell_times_mat(G, V');
        g = sum(U .* H)' + lambda;
      else
        % do fancy
        [f, g] = ctx_total.obj_total_correct(x, ctx_total);
        f = f + lambda*sum(x);
        g = g + lambda;
      end
    end  
end
